/* -*- mode: c++; c-basic-offset: 4 -*- */

#include "subprocess.h"

#include <iostream>
#include <string>
#include <utility>
#include <random>
#include <vector>
#include <cstdint>
#include <cassert>
#include <sstream>
#include <array>
#include <fstream>

std::mt19937 random_gen;

bool ends_with(const std::string &str, const std::string &suffix)
{
    if (suffix.length() > str.length()) return false;
    return str.compare(str.length() - suffix.length(), suffix.length(), suffix) == 0;
}

std::pair<bool, std::string> invoke(const std::vector<std::string> &args, const std::string &data)
{
    Subprocess sub;

    sub.set_cmd(args[0]);
    for (int i = 1; i < int(args.size()); ++i) {
        sub.add_arg(args[i]);
    }

    sub.set_input(data);
    sub.set_timeout(1000);

    {
        std::ofstream ofs("input.txt");
        ofs << data;
    }

    bool result = sub.run_and_wait();

    std::cerr << "err: " << sub.error() << std::endl;

    if (!result) return { false, {}};

    {
        std::ifstream ifs("output.txt");
        if (ifs) {
            std::string buf;
            char c;
            while (ifs.get(c)) {
                buf += c;
            }
            return { true, sub.output() + buf };
        }
    }

    return { true, sub.output() };
}

struct Node
{
    int serial{};
    int32_t id{};
    std::vector<int> edges;
    int state = -1;

    Node(int serial, int32_t id) : serial(serial), id(id) {}

    void add_edge(int serial)
    {
        edges.push_back(serial);
    }
};

struct Edge
{
    int serial{};
    int node1{};
    int node2{};

    std::array<int64_t, 2> input;
    std::array<int64_t, 2> output;

    Edge(int serial, int node1, int node2) : serial(serial), node1(node1), node2(node2) {}

    void send(int from, int64_t value)
    {
        if (from == node1) {
            output[0] = value;
        } else if (from == node2) {
            output[1] = value;
        } else {
            abort();
        }
    }

    int64_t recv(int to)
    {
        if (to == node1) {
            return input[0];
        } else if (to == node2) {
            return input[1];
        } else {
            abort();
        }
    }

    void copy_msgs()
    {
        input[1] = output[0];
        input[0] = output[1];
        output[0] = 0;
        output[1] = 0;
    }
};

struct Graph
{
    std::vector<Node> nodes;
    std::vector<Edge> edges;

    void add_node(int32_t id)
    {
        nodes.emplace_back(nodes.size(), id);
    }

    void add_edge(int node1, int node2)
    {
        int n1 = std::min(node1, node2);
        int n2 = std::max(node1, node2);

        assert(n1 >= 0 && n1 < int(nodes.size()));
        assert(n2 >= 0 && n2 < int(nodes.size()));
        if (n1 == n2) {
            std::cerr << "edge to itself " << n1 << " " << n2 << std::endl;
            exit(1);
        }

        for (const auto &e : edges) {
            if (e.node1 == n1 && e.node2 == n2) {
                std::cerr << "duplicated edge " << n1 << " " << n2 << std::endl;
                exit(1);
            }
        }

        int serial = edges.size();
        edges.emplace_back(serial, n1, n2);
        nodes[n1].add_edge(serial);
        nodes[n2].add_edge(serial);
    }

    void send(int src_node, int edge_index, int64_t value)
    {
        assert(src_node >= 0 && src_node < int(nodes.size()));
        auto &node = nodes[src_node];
        assert(edge_index >= 0 && edge_index < int(node.edges.size()));
        edges[node.edges[edge_index]].send(src_node, value);
    }

    int64_t recv(int dst_node, int edge_index)
    {
        assert(dst_node >= 0 && dst_node < int(nodes.size()));
        auto &node = nodes[dst_node];
        assert(edge_index >= 0 && edge_index < int(node.edges.size()));
        return edges[node.edges[edge_index]].recv(dst_node);
    }

    void copy_msgs()
    {
        for (auto &edge : edges) {
            edge.copy_msgs();
        }
    }
};

int main(int argc, char *argv[])
{
    std::vector<std::string> botargs;
    for (int i = 1; i < argc; ++i) {
        botargs.push_back(argv[i]);
    }

    uint32_t seed;
    if (!(std::cin >> seed)) {
        std::cerr << "failed to read seed" << std::endl;
        exit(1);
    }
    random_gen.seed(seed);

    Graph graph;

    int nodes;
    std::cin >> nodes;
    if (!std::cin || nodes < 1 || nodes > 10) {
        std::cerr << "failed to read nodes" << std::endl;
        exit(1);
    }
    for (int i = 0; i < nodes; ++i) {
        graph.add_node(random_gen() & 0x7fffffff);
    }

    for (int i = 0; i < nodes; ++i) {
        int edgcnt;
        if (!(std::cin >> edgcnt)) {
            std::cerr << "failed to read edge count " << i << std::endl;
            exit(1);
        }
        if (edgcnt < 0 || edgcnt > 100) {
            std::cerr << "failed to read edge count " << i << std::endl;
            exit(1);
        }
        for (int j = 0; j < edgcnt; ++j) {
            int dest;
            if (!(std::cin >> dest)) {
                std::cerr << "failed to read edge" << std::endl;
                exit(1);
            }
            if (dest < 0 || dest >= nodes) {
                std::cerr << "failed to read edge" << std::endl;
                exit(1);
            }
            graph.add_edge(i, dest);
        }
    }
    char c;
    if ((std::cin >> c)) {
        std::cerr << "garbage after data" << std::endl;
        exit(1);
    }

    int round = 0;
    while (1) {
        ++round;

        if (round == 51) {
            std::cerr << "too many rounds" << std::endl;
            exit(1);
        }

        std::cerr << "round: " << round << std::endl;
        std::cerr << "nodes:";
        for (const auto &node : graph.nodes) {
            std::cerr << " " << node.id << ":" << node.state;
        }
        std::cerr << std::endl;

        for (auto &node : graph.nodes) {
            if (node.state != -1) continue;

            std::ostringstream oss;
            oss << node.id << std::endl;
            oss << node.edges.size() << std::endl;
            for (int i = 0; i < int(node.edges.size()); ++i) {
                oss << graph.recv(node.serial, i) << std::endl;
            }
            std::cerr << "node " << node.id << ":" << std::endl;
            std::cerr << "input: <" << oss.str() << ">" << std::endl;

            auto [ result, out ] = invoke(botargs, oss.str());
            if (!result) {
                std::cerr << "invocation failed for " << botargs[0] << std::endl;
                exit(1);
            }

            std::cerr << "output: <" << out << ">" << std::endl;
            std::istringstream iss{out};
            int state;
            if (!(iss >> state)) {
                std::cerr << "failed to parse new state" << std::endl;
                exit(1);
            }
            if (state != -1 && state != 0 && state != 1) {
                std::cerr << "invalid new state" << std::endl;
                exit(1);
            }
            node.state = state;
            if (state == -1) {
                for (int i = 0; i < int(node.edges.size()); ++i) {
                    int64_t msg;
                    if (!(iss >> msg)) {
                        std::cerr << "failed to read message" << std::endl;
                        exit(1);
                    }
                    graph.send(node.serial, i, msg);
                }
            }
            char c;
            if ((iss >> c)) {
                std::cerr << "garbage after messages" << std::endl;
                exit(1);
            }
        }

        int winners = 0;
        int losers = 0;
        int runners = 0;
        for (const auto &node : graph.nodes) {
            if (node.state == -1) {
                ++runners;
            } else if (node.state == 0) {
                ++losers;
            } else if (node.state == 1) {
                ++winners;
            } else {
                abort();
            }
        }

        if (!runners) {
            if (winners > 1) {
                std::cerr << "too many winners" << std::endl;
                exit(1);
            } else if (!winners) {
                std::cerr << "no winners" << std::endl;
                exit(1);
            } else {
                std::cerr << "winner selected, " << round << " turns" << std::endl;
                std::cout << 1 << std::endl;
                exit(0);
            }
        }

        graph.copy_msgs();
    }
}


